/*********************************************************************/
/*                                                                   */
/* Check a compiled Maximus 3.0 MEX program to see if it will run in */
/* DOS.                                                              */
/*                                                                   */
/* This program simply opens up the .VM file and checkes if the word */
/*   at offset x'14' is less than x'0b9b'.                           */
/* If so it prints an "OK" message and exits with errorlevel 0       */
/* Otherwise it prints a message telling how much too big it is and  */
/*   exits with errorlevel 1.                                        */
/* If any problems are encountered a error message is printed and    */
/*   we exit with error level 2.                                     */
/*                                                                   */
/*********************************************************************/

char copyright[] = {"Copyright (c) 1995 Don Reid, All Rights Reserved."};

#include <stdio.h>
#include <dir.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <share.h>
#include <string.h>
#include <stdlib.h>

/* The following constants were determined from reverse-enginering a number
   of .VM files */

#define CODEOVERHEAD 512
#define QUADSIZE      22
#define QUADLOC       20

#define VERSION "1.0"
#define PROGRAM "OK4DOS"
#define PGMDESC "Check a compiled .MEX to see if it will run under DOS"
#define COPYRIGHT "1995"
#define AUTHOR "Don Reid"
#define ADDRESS "1:342/80"



/****************************************************************************
 *         Function PrintInfo  -  prints braglines                          *
 ****************************************************************************/
void PrintInfo(char * program, char * pgmdesc, char * version,
               char * copyright, char * author, char * address)
{
    printf("%8s: %s Ver. %s\n"
           "          Copyright (C) %s by %s of %s. All rights reserved.\n",
           program, pgmdesc, version,
           copyright, author, address);
}




/***************************************************************************
 *        Function PrintHelp  -  prints help screen                        *
 ***************************************************************************/
void PrintHelp(char * program)
{
    printf("\
Syntax:\n\
      %s [-q] <filename>\n\n\
      -q         - If specified supresses the copyright banner\n\
      <filename> - path/name of Maximus 3.0 MEX program to check\n\
                   The extension if omited is assumed to be .VM.\n\
                   If .MEX is specified for the extension, .VM is used\n\n\
Results:\n\
      ErrorLevel  Meaning\n\
          0       Will run under DOS\n\
          1       Too big to run under DOS\n\
          2       Error\n\n",program);
}



int main(int argc, char * argv[])
{
    struct {
        char filler1[QUADLOC];
        int  quads;
        char filler2[512-(QUADLOC+2)];
    } FileStart;

    long  codesize;
    int   VMFile;
    int   bytes_read;

    int   quiet = {0};
    int   filenameindex = {1};

    char ShortName[MAXPATH];
    char FileName[MAXPATH];

    char drive[MAXDRIVE];
    char dir[MAXDIR];
    char name[MAXFILE];
    char ext[MAXEXT];

    if (argc == 3)
    {
        if (!strcmpi(argv[1],"-q"))
        {
            quiet = 1;
            filenameindex = 2;
        } else
        {
            PrintInfo(PROGRAM,PGMDESC,VERSION,
                      COPYRIGHT,AUTHOR,ADDRESS);
            fprintf(stderr,"\nInvalid arguments `%s %s'. :-(\n\n",
                    argv[1],argv[2]);
            PrintHelp(PROGRAM);
            return 2;
        }
    } else if (argc != 2)
    {
        PrintInfo(PROGRAM,PGMDESC,VERSION,
                  COPYRIGHT,AUTHOR,ADDRESS);
        fprintf(stderr,"\nWrong number of arguments specified :-(\n\n");
        PrintHelp(PROGRAM);
        return 2;
    }

    if (!quiet)
    {
        PrintInfo(PROGRAM,PGMDESC,VERSION,
                  COPYRIGHT,AUTHOR,ADDRESS);
    }

    fnsplit(argv[filenameindex],drive,dir,name,ext);

    if (!*ext)
    {
        strcpy(ext,".VM");
    } else if (!ext[1])
    {
        strcpy(ext,".VM");
    } else if (!strcmpi(".MEX",ext))
    {
        strcpy(ext,".VM");
    }

    fnmerge(ShortName,drive,dir,name,ext);
    _fullpath(FileName,ShortName,sizeof(FileName));

    fprintf(stderr,"\nProcessing `%s'.\n",FileName);

    if (0 > (VMFile=sopen(FileName,O_RDONLY|O_BINARY,SH_DENYWR)))
    {
        fprintf(stderr,"Error in trying to open. :-(\n");
        return 2;
    }

    bytes_read = read(VMFile,&FileStart,sizeof(FileStart));
    close(VMFile);

    if (bytes_read != sizeof(FileStart))
    {
        fprintf(stderr,"Error reading. :-(\n");
        return 2;
    }

    codesize = (long)CODEOVERHEAD + ((long)FileStart.quads * (long)QUADSIZE);

    if (codesize > 65535)
    {
        fprintf(stderr,"Code size = %li, which is too big for DOS :-(\n",
                codesize);
        return 1;
    } else
    {
        fprintf(stderr,"Code size = %li. This will run in DOS :-)\n",
                codesize);
    }

    return 0;
}
